@@ -13,6 +13,8 @@ Welcome to PuDB, the Python Urwid debugger.
 -------------------------------------------
 
 Keys:
+    Ctrl-p - edit preferences
+
     n - step over ("next")
     s - step into
     c - continue
@@ -20,6 +22,9 @@ Keys:
     t - run to cursor
     e - show traceback [post-mortem or in exception state]
 
+    u - move up one stack frame
+    d - move down one stack frame
+
     ! - invoke python shell in current environment
     o - show console/output screen
 
@@ -114,6 +119,7 @@ class Debugger(bdb.Bdb):
         self.bottom_frame = None
         self.mainpyfile = ''
         self._wait_for_mainpyfile = False
+        self.current_bp = None
         self.post_mortem = False
 
     def restart(self):
@@ -132,6 +138,14 @@ class Debugger(bdb.Bdb):
         self.ui.update_var_view()
         self.ui.update_stack()
 
+    def move_up_frame(self):
+        if self.curindex > 0:
+            self.set_frame_index(self.curindex-1)
+
+    def move_down_frame(self):
+        if self.curindex < len(self.stack)-1:
+            self.set_frame_index(self.curindex+1)
+
     def get_shortened_stack(self, frame, tb):
         stack, index = self.get_stack(frame, tb)
 
@@ -195,6 +209,12 @@ class Debugger(bdb.Bdb):
             self._wait_for_mainpyfile = False
             self.bottom_frame = frame
 
+        if self.get_break(self.canonic(frame.f_code.co_filename), frame.f_lineno):
+            self.current_bp = (self.canonic(frame.f_code.co_filename), frame.f_lineno)
+        else:
+            self.current_bp = None
+        self.ui.update_breakpoints()
+
         self.interaction(frame)
 
     def user_return(self, frame, return_value):
@@ -242,7 +262,7 @@ class Debugger(bdb.Bdb):
 
 # UI stuff --------------------------------------------------------------------
 from pudb.ui_tools import make_hotkey_markup, labelled_value, \
-        SelectableText, SignalWrap, StackFrame, \
+        SelectableText, SignalWrap, StackFrame, BreakpointFrame, \
         SearchBox
 from pudb.var_view import FrameVarInfoKeeper
 
@@ -328,6 +348,9 @@ class DebuggerUI(FrameVarInfoKeeper):
         def edit_inspector_detail(w, size, key):
             var, pos = self.var_list._w.get_focus()
 
+            if var is None:
+                return
+
             fvi = self.get_frame_var_info(read_only=False)
             iinfo = fvi.get_inspect_info(var.id_path, read_only=False)
 
@@ -435,9 +458,23 @@ class DebuggerUI(FrameVarInfoKeeper):
 
         self.stack_list.listen("enter", examine_frame)
 
+        def move_stack_up(w, size, key):
+            self.debugger.move_up_frame()
+        def move_stack_down(w, size, key):
+            self.debugger.move_down_frame()
+
+        self.stack_list.listen("u", move_stack_up)
+        self.stack_list.listen("d", move_stack_down)
+        self.source_sigwrap.listen("u", move_stack_up)
+        self.source_sigwrap.listen("d", move_stack_down)
+
         # stack listeners -----------------------------------------------------
         def examine_breakpoint(w, size, key):
-            _, pos = self.bp_list._w.get_focus()
+            bp_entry, pos = self.bp_list._w.get_focus()
+
+            if bp_entry is None:
+                return
+
             bp = self._get_bp_list()[pos]
 
             if bp.cond is None:
@@ -842,50 +879,6 @@ class DebuggerUI(FrameVarInfoKeeper):
                 self.message("No exception available.")
 
         def run_shell(w, size, key):
-            import pudb.shell as shell
-
-            if shell.HAVE_IPYTHON and shell.USE_IPYTHON == "ask":
-                def ipython(w, size, key):
-                    self.quit_event_loop = ["ipython"]
-                def classic(w, size, key):
-                    self.quit_event_loop = ["classic"]
-                def cancel(w, size, key):
-                    self.quit_event_loop = [False]
-
-                result = dbg.ui.dialog(
-                    urwid.ListBox([urwid.Text(
-                        "You've asked to enter a Python shell.\n\n"
-                        "You appear to have IPython installed. If you wish to use it, "
-                        "you may hit 'i' or select the corresponding button on the "
-                        "right. If you prefer the 'classic' Python shell, hit '!' again or "
-                        "select that button instead.\n\n"
-                        "Sorry for bothering you, I won't ask again in this session."
-                        )]),
-                    [
-                        ("Classic", "classic"),
-                        ("IPython", "ipython"),
-                        ("Cancel", False),
-                        ],
-                    focus_buttons=True,
-                    bind_enter_esc=False,
-                    title="Shell Requested",
-                    extra_bindings=[
-                        ("!", classic),
-                        ("i", ipython),
-                        ("esc", cancel),
-                        ])
-
-                if result == False:
-                    return
-                elif result == "ipython":
-                    shell.USE_IPYTHON = use_ipython = True
-                elif result == "classic":
-                    shell.USE_IPYTHON = use_ipython = False
-
-            elif shell.HAVE_IPYTHON and shell.USE_IPYTHON:
-                use_ipython = True
-            else:
-                use_ipython = False
 
             self.screen.stop()
 
@@ -897,12 +890,14 @@ class DebuggerUI(FrameVarInfoKeeper):
 
             curframe = self.debugger.curframe
 
-            if use_ipython:
+            from pudb import CONFIG
+            import pudb.shell as shell
+            if shell.HAVE_IPYTHON and CONFIG["shell"] == "ipython":
                 runner = shell.run_ipython_shell
             else:
                 runner = shell.run_classic_shell
 
-            runner(curframe.f_locals, curframe.f_globals, 
+            runner(curframe.f_locals, curframe.f_globals,
                     first_shell_run)
 
             self.screen.start()
@@ -937,9 +932,20 @@ class DebuggerUI(FrameVarInfoKeeper):
             self.debugger.set_quit()
             end()
 
+        def do_edit_config(w, size, key):
+            from pudb.settings import edit_config, save_config
+            from pudb import CONFIG
+            edit_config(self, CONFIG)
+            save_config(CONFIG)
+            self.setup_palette(self.screen)
+
+            for sl in self.source:
+                sl._invalidate()
+
         def help(w, size, key):
             self.message(HELP_TEXT, title="PuDB Help")
 
+
         self.top.listen("o", show_output)
         self.top.listen("!", run_shell)
         self.top.listen("e", show_traceback)
@@ -951,6 +957,7 @@ class DebuggerUI(FrameVarInfoKeeper):
         self.top.listen("B", RHColumnFocuser(2))
 
         self.top.listen("q", quit)
+        self.top.listen("ctrl p", do_edit_config)
         self.top.listen("H", help)
         self.top.listen("f1", help)
         self.top.listen("?", help)
@@ -1040,8 +1047,10 @@ class DebuggerUI(FrameVarInfoKeeper):
         may_use_fancy_formats = isinstance(screen, RawScreen) and \
                 not hasattr(urwid.escape, "_fg_attr_xterm")
 
+        from pudb import CONFIG
         from pudb.theme import get_palette
-        screen.register_palette(get_palette(may_use_fancy_formats))
+        screen.register_palette(
+                get_palette(may_use_fancy_formats, CONFIG["theme"]))
 
     # UI enter/exit -----------------------------------------------------------
     def show(self):
@@ -1073,6 +1082,30 @@ class DebuggerUI(FrameVarInfoKeeper):
                 self.message("Package 'pygments' not found. "
                         "Syntax highlighting disabled.")
 
+        from pudb import CONFIG
+        WELCOME_LEVEL = "b"
+        if CONFIG["seen_welcome"] < WELCOME_LEVEL:
+            CONFIG["seen_welcome"] = WELCOME_LEVEL
+            from pudb import VERSION
+            self.message("Welcome to PudB %s!\n\n"
+                    "PuDB is a full-screen, console-based visual debugger for Python. "
+                    " Its goal is to provide all the niceties of modern GUI-based "
+                    "debuggers in a more lightweight and keyboard-friendly package. "
+                    "PuDB allows you to debug code right where you write and test it--in "
+                    "a terminal. If you've worked with the excellent (but nowadays "
+                    "ancient) DOS-based Turbo Pascal or C tools, PuDB's UI might "
+                    "look familiar.\n\n"
+                    "New features in version 0.93:\n\n"
+                    "- Stored preferences (no more pesky IPython prompt!)\n"
+                    "- Themes\n"
+                    "- Line numbers (optional)\n"
+                    "\nHit Ctrl-P to set up PuDB.\n\n"
+                    "If you're new here, welcome! The help screen (invoked by hitting "
+                    "'?' after this message) should get you on your way." % VERSION)
+
+            from pudb.settings import save_config
+            save_config(CONFIG)
+
         try:
             if toplevel is None:
                 toplevel = self.top
@@ -1102,8 +1135,8 @@ class DebuggerUI(FrameVarInfoKeeper):
 
         from pudb import VERSION
         caption = [(None,
-            u"PuDB %s - The Python Urwid debugger - Hit ? for help"
-            u" - (C) Andreas Kloeckner 2009"
+            u"PuDB %s - ?:help, n:next, s:step into, b:breakpoint, o:console,"
+            "t:run to cursor, !:python shell"
             % VERSION)]
 
         if self.debugger.post_mortem:
@@ -1218,13 +1251,9 @@ class DebuggerUI(FrameVarInfoKeeper):
         return name
 
     def update_breakpoints(self):
-        def format_bp(bp):
-            return "%s:%d" % (self._format_fname(bp.file), bp.line)
-
         self.bp_walker[:] = [
-                urwid.AttrWrap(
-                    SelectableText(format_bp(bp), wrap="clip"),
-                    None, "focused breakpoint")
+                BreakpointFrame(self.debugger.current_bp == (bp.file, bp.line),
+                    self._format_fname(bp.file), bp.line)
                 for bp in self._get_bp_list()]
 
     def update_stack(self):
